String CACHE_FILENAME = "cache.txt";


class FlixsterAPI {
  String apiKey;
  int pageLimit = 10;
  String hostname = "api.rottentomatoes.com";
  String apiPath = "/api/public/v1.0/";
  HashMap memo;
  JSONObject cache;
  
  FlixsterAPI(String api) {
    apiKey = api;
    memo = new HashMap(); // Used for the levenshteinDistance
    
    String[] lines = loadStrings(CACHE_FILENAME);
    if (lines == null)
      return;
      
    try {
      cache = new JSONObject(lines[0]);      
    } catch(Exception JSONException) {
      println("Error parsing the JSON in the cache.txt file");
      println(JSONException);
    }
   
  }
  
  JSONObject searchMovie(String title) {
    
    println(title);
    
    if (cache != null) {
      println("Checking the cache");
      
      try {
        
        JSONObject movie = cache.getJSONObject(title); // might need to also search by year..but how to store in cache.
        println(":) Found movie in cache");
        return movie;
        
      } catch(Exception JSONException) {
        // Fall through and actually perform the requests to Flixster
      }      
      
    }
    
    println("!! Querying flixster for movie");
    
    URI uri;
    try {
       uri = new URI(
        "http",
        hostname,
        apiPath + "movies.json",
        "q=" + title + "&page_limit=" + pageLimit + "&apiKey=" + apiKey,
        null);
    } catch (Exception URISyntaxException) {
      println("some URI exception");
      return null;
    }
            
    String response = join(loadStrings(uri.toASCIIString()), "");
    
    JSONObject movieData, bestMatch = null;
    
    try {
      movieData = new JSONObject(response);
      JSONArray movies = movieData.getJSONArray("movies");
      

      if (movies.length() == 0) {
        return null;
      } else if (movies.length() == 1) {
        bestMatch = movies.getJSONObject(0);
      }
      else {
        int min_l_dist = Integer.MAX_VALUE;
       
        for (int i = 0; i < movies.length(); i++) {
          JSONObject movie = movies.getJSONObject(i);
          String rottenMovieTitle = movie.getString("title");
          int l_dist = levenshteinDistance(title, rottenMovieTitle); 
          
          if (l_dist == min_l_dist) {
            try {
              int year = movie.getInt("year");
              int bestMatchYear = bestMatch.getInt("year");
              if (year > bestMatchYear) {
                bestMatch = movie;
              }
            } catch (Exception JSONException) {
            }
          }
          else if (l_dist < min_l_dist) {
            min_l_dist = l_dist;
            bestMatch = movie;
          }
        }
      }
    }
    catch (Exception JSONException) {
      
      println(JSONException);
      println("Error parsing JSON");
      return null;
    }
     
    
    //Movie movie = new Movie(bestMatch.getString("title"), bestMatch.getJSONObject("posters").getString("profile"));
    try {
      cache.put(title, bestMatch);
    } catch(Exception JSONException) {
      println("Error putting bestMatch in cache");
      println(JSONException);
    }
    
    return bestMatch;
  }
  
  void saveCache() {
    String[] strings = { cache.toString() };
    saveStrings(CACHE_FILENAME, strings);
  }
  
  int levenshteinDistance(String str1, String str2) {
    return _levenshteinDistance(str1.toLowerCase(), 0, str1.length(), str2.toLowerCase(), 0, str2.length());
  }
  
  int _levenshteinDistance(String str1, int i, int len1, String str2, int j, int len2) {
    
    String strKey = i + "," + len1 + "," + j + "," + len2;
    
    if (memo.containsKey(strKey)) 
      return (Integer) memo.get(strKey);
    
    if (len1 == 0) return len2;
    if (len2 == 0) return len1;
    
    int cost = 0;
    if (str1.charAt(i) != str2.charAt(j))
      cost = 1;
      
    int dist = Math.min(Math.min(_levenshteinDistance(str1, i+1, len1-1, str2, j, len2)+1,
               _levenshteinDistance(str1, i, len1, str2, j+1, len2-1)+1), 
               _levenshteinDistance(str1, i+1, len1-1, str2, j+1, len2-1) + cost);
               
    memo.put(strKey, dist);
    return dist;
  }
}

